create or replace PACKAGE      "DATACONS_SFCONNECTOR" 
as
/**********************************************************************************
 * This type is returned by query function. Every row from Snowflake is returned as JSON 
 **********************************************************************************/
  type T_JSON_TAB is table of clob;

/**********************************************************************************
 * Before any other activity you have to login to Snowflake account. 
 **********************************************************************************/
  procedure login(pi_account varchar2, pi_user varchar2, pi_password varchar2, pi_wallet varchar2);

/**********************************************************************************
 * This procedure executes statement in Snowflake DWH
 **********************************************************************************/
  procedure execute(pi_stmt   in varchar2);

/**********************************************************************************
 * This procedure executes statement in Snowflake DWH allowing you to use bind variables stored in a table. 
 * You can execute multi-row INSERT using that procedure.
 **********************************************************************************/
  procedure execute(pi_stmt   in varchar2,
                    pi_binds  in json_array_t);

/**********************************************************************************
 * This procedure executes statement in Snowflake DWH allowing you to use bind variables
 **********************************************************************************/
  procedure execute(pi_stmt   in varchar2,
                    pi_binds  in json_key_list);

/**********************************************************************************
 * This procedure executes statement in Snowflake DWH allowing you to read output of the query. 
 * You can optionaly pass bind variables to the query. 
 * By default function is waiting for query to return result. 
 * If you want to run your query asynchronously - pass pi_wait_for_result 0
 * You can retrive results of query run in asynchronous mode (after it is finished) using pi_query_id paramater
 **********************************************************************************/
  function query(pi_query             in varchar2 default null,
                 pi_binds             in json_key_list default null,
                 pi_wait_for_result   in varchar2 default 1,
                 pi_query_id          in varchar2 default null) return t_json_tab  pipelined;

/**********************************************************************************
 * This function allow you to read status of query that is run in asynchronous mode
 **********************************************************************************/
 function get_query_status(pi_query_id  in varchar2) return varchar2;

/**********************************************************************************
 * Sets maximum time to wait for query function when pi_wait_for_result is set to 0
 **********************************************************************************/
  procedure set_max_wait_for_async_query(pi_seconds in pls_integer);

/**********************************************************************************
 * Returns maximum time to wait for query function when pi_wait_for_result is set to 0
 **********************************************************************************/
  function get_max_wait_for_async_query return pls_integer;

/**********************************************************************************
 * Set verbosity of Connector
 **********************************************************************************/
procedure set_verbose(pi_value in pls_integer);

/**********************************************************************************
 * Get verbosity of Connector
 **********************************************************************************/
function get_verbose return pls_integer;

end DATACONS_SFCONNECTOR;